/*
 * $Id: re.c,v 1.4 2001/12/20 15:01:36 kenta Exp $
 *
 * Copyright 2001 Kenta Cho. All rights reserved.
 */

/**
 * Re: W32/Badtrance boot strap.
 *
 * @version $Revision: 1.4 $
 */
#include <string.h>
#include <stdlib.h>
#include <piece.h>

#include "re.h"
#include "gamemanager.h"
#include "attractmanager.h"
#include "render.h"

unsigned char vbuf[SCREEN_WIDTH*SCREEN_HEIGHT];

// sounds
static PCEWAVEINFO wtemp;
static char *sndBuf[4];

// images
extern unsigned char RE1[], RE2[], RE3[], RE4[], OL[], PL[], TRG[], NLK[],
  ZER[], ONE[], TWO[], THR[], FOU[], FIV[], SIX[], SEV[], EIG[], NIN[],
  TTL[], START[], GOV[], SCORE[];
PIECE_BMP re1Bmp, re2Bmp, re3Bmp, re4Bmp, olBmp, plBmp, trgBmp, nlkBmp, numBmp[10],
  titleBmp, startBmp, govBmp, scrBmp;

int status;
int cnt;

static int bgmNum;
static int changeStage;

static int deadCnt;

int demo;

void clearScreen() {
  memset(vbuf, 0, SCREEN_WIDTH*SCREEN_HEIGHT);
}

static void stopSe() {
  int i;
  for ( i=0 ; i<4 ; i++ ) {
    pceWaveAbort(i);
    sndBuf[i] = NULL;
  }
}

void initTitle() {
  status = GAME_TITLE; cnt = -1;
  stopSe();
}

void initInGame() {
  status = IN_GAME; cnt = -1;
  bgmNum = 0;
  changeStage = 0;
  demo = 0;
  initGame(0);
}

void initDemo() {
  status = IN_GAME; cnt = -1;
  bgmNum = randN(4);
  changeStage = 0;
  demo = 1;
  initGame(bgmNum);
}

void initPlayerDead() {
  if ( demo ) {
    initTitle();
  } else {
    status = DEAD; deadCnt = 0;
  }
  stopSe();
}

void initGameover() {
  status = GAME_OVER; cnt = -1;
}

static void loadBmp(unsigned char *data, PIECE_BMP *bmp) {
  PBMP_FILEHEADER pbhead;
  memcpy(&pbhead, data, sizeof(PBMP_FILEHEADER));
  bmp->header = pbhead;
  bmp->buf  = data + sizeof(PBMP_FILEHEADER);
  bmp->mask = data + sizeof(PBMP_FILEHEADER) + (bmp->header.h*bmp->header.w)/4;
}

#define INTERVAL 31

void pceAppInit() {
  PCETIME time;

  //while(pcePadGet()) pcePadGetProc();
  //pcePadSetTrigMode(PP_MODE_SINGLE);

  //loadHiScore();

  pceLCDDispStop();
  pceLCDSetBuffer(vbuf);

  loadBmp(RE1, &re1Bmp);
  loadBmp(RE2, &re2Bmp);
  loadBmp(RE3, &re3Bmp);
  loadBmp(RE4, &re4Bmp);
  loadBmp(OL, &olBmp);
  loadBmp(PL, &plBmp);
  loadBmp(TRG, &trgBmp);
  loadBmp(NLK, &nlkBmp);
  loadBmp(ZER, &numBmp[0]);
  loadBmp(ONE, &numBmp[1]);
  loadBmp(TWO, &numBmp[2]);
  loadBmp(THR, &numBmp[3]);
  loadBmp(FOU, &numBmp[4]);
  loadBmp(FIV, &numBmp[5]);
  loadBmp(SIX, &numBmp[6]);
  loadBmp(SEV, &numBmp[7]);
  loadBmp(EIG, &numBmp[8]);
  loadBmp(NIN, &numBmp[9]);
  loadBmp(TTL, &titleBmp);
  loadBmp(START, &startBmp);
  loadBmp(GOV, &govBmp);
  loadBmp(SCORE, &scrBmp);
  
  pceTimeGet(&time);
  srand(time.s100 + time.ss*100);

  sndBuf[0] = sndBuf[1] = sndBuf[2] = sndBuf[3] = NULL;

  initGamemanager();

  initTitle();

  pceAppSetProcPeriod(INTERVAL);
  pceLCDDispStart();
}

void playSe(char *snd, int ch) {
  sndBuf[ch] = snd;
} 

static void playAllSound() {
  int i;
  for ( i=0 ; i<4 ; i++ ) {
    if ( sndBuf[i] == NULL ) continue;
    wtemp = *((PCEWAVEINFO*)(sndBuf[i] + 8));
    wtemp.pData = sndBuf[i] + 8 + sizeof(PCEWAVEINFO);
    pceWaveDataOut(i, &wtemp);
    sndBuf[i] = NULL;
  }
}

static void drawScene() {
  int pad = pcePadGet();
  int ren;

  if ( (cnt%128) == 0 ) {
    if ( changeStage == 2 ) {
      changeStage = 0;
      bgmNum++; if ( bgmNum > 4 ) bgmNum =0;
      playSe(HIT, 3);
      gotoNextStage();
    } else if ( changeStage == 1 ) {
      changeStage = 2;
      playSe(FILL, 3);
    }
    switch ( bgmNum ) {
    case 0:
      playSe(BGM1, 0);
      break;
    case 1:
      playSe(BGM2, 0);
      break;
    case 2:
      playSe(BGM3, 0);
      break;
    case 3:
      playSe(BGM4, 0);
      break;
    case 4:
      playSe(BGM5, 0);
      break;
    }
  }
  if ( changeStage == 2 ) {
    setGroundVertices((stage+1)%5, stage%5, (cnt%128)*2);
  }

  if ( (cnt&3) == 0 ) playAllSound();

  clearScreen();
  drawGround();
  ren = drawRe();
  drawBullet();
  drawPlayer();
  drawLock();

  if ( stageReNum <= 0 && ren <= 2 && changeStage == 0 ) changeStage = 1;

  if ( demo && ( cnt > 960 || (pad & (TRG_A | TRG_B | TRG_C) ) ) ) initTitle();
}

static void drawDeadScene() {
  clearScreen();
  drawGround();
  drawRe();
  drawPlayer();
  drawBullet();
  setGroundVertices(5, stage%5, deadCnt*2);
  deadCnt++;
  if ( deadCnt >= 128 ) {
    //if ( score > hiScore ) hiScore = score;
    initGameover();
  }
}

void pceAppProc(int c) {
  int pad = pcePadGet();

  switch ( status ) {
  case GAME_TITLE:
    drawTitle();
    break;
  case IN_GAME:
    drawScene();
    break;
  case DEAD:
    drawDeadScene();
    break;
  case GAME_OVER:
    drawGameover();
    break;
  }
  cnt++;

  //**
  if ( pad & TRG_D ) {
    pceAppReqExit(1);
  }
  pceLCDTrans();
}

void pceAppExit( void ) {
  stopSe();
  //saveHiScore();
}
